==> zreact-0.3.0.tgz: package/src/devtools-base.ts <==
import { options, Component, findDOMNode as IfindDOMNode, findVDom as IfindVDom} from "./zreact";
import { IKeyValue } from "./types";
import { IVDom } from "./vdom/index";

interface IReactElement {
    props?: boolean|IKeyValue;
    type: any;
    ref?: ((c: Component<IKeyValue, IKeyValue>|null) => void)|null;
    key?: string;
}
interface IReactComponent {
    _currentElement: IReactElement|string|null;
    node?: Text|Element|Node;
    _instance?: Component<IKeyValue, IKeyValue>;
    _renderedComponent?: IReactComponent;
    forceUpdate?: () => void;
    getName?: () => void;
    props?: IKeyValue;
    setState?: () => void;
    state?: IKeyValue;
    _inDevTools?: boolean;
    _renderedChildren?: IReactComponent[];
    _stringText?: null|string;
    _rootID?: string;
    _component?: any;
}

declare const window: {
    $zreact: IVDom,
    __REACT_DEVTOOLS_GLOBAL_HOOK__?: any,
};

declare class Map {
    public has(key: any): boolean;
    public get(key: any): any;
    public set(key: any, val: any): void;
    public delete(key: any): void;
}

export function getInitDevTools(opt: typeof options, findDOMNode: typeof IfindDOMNode, findVDom: typeof IfindVDom) {

    /**
     * 将zreact的component实例转换为React的
     * @param  component
     */
    function createReactElement(component: Component<IKeyValue, IKeyValue>): IReactElement {
        const element: IReactElement = {
            key: component._key,
            props: component.props,
            ref: null, // Unsupported
            type: component.constructor,
        };
        return element;
    }

    /**
     * 获取组件名
     * @param  element
     */
    function typeName(element: IReactElement): string {
        if (typeof element.type === "function") {
            return element.type.displayName || element.type.name;
        }
        return element.type;
    }
    /**
     * 将component转换实例
     * @param component
     */
    function createReactCompositeComponent(component: Component<IKeyValue, IKeyValue>): IReactComponent {
        const _currentElement = createReactElement(component);
        const node = findDOMNode(component);
        const instance: IReactComponent = {
            _currentElement,
            _instance: component,
            forceUpdate: component.forceUpdate && component.forceUpdate.bind(component),
            getName: function getName() {
                return typeName(_currentElement);
            },
            node,
            props: component.props,
            setState: component.setState && component.setState.bind(component),
            state: component.state,
        };
        if (component._component) {
            instance._renderedComponent = updateReactComponent(component._component);
        } else if (findVDom(component)) {
            instance._renderedComponent = updateReactComponent(findVDom(component));
        }
        return instance;
    }

    /**
     * 将vdom转换为实例
     * @param vdom
     */
    function createReactDOMComponent(vdom: IVDom): IReactComponent {
        const node = findDOMNode(vdom);
        const childNodes = node.nodeType === Node.ELEMENT_NODE ?
            Array.prototype.slice.call(node.childNodes, 0) : [];
        const isText = node.nodeType === Node.TEXT_NODE;
        let element: string|IReactElement|null;
        if (isText) {
            element = node.textContent;
        } else {
            element = {
                props: vdom.props,
                type: node.nodeName.toLowerCase(),
            };
        }
        const children: IReactComponent[] = new Array();
        childNodes.forEach(function _(child: any) {
            const childVDom = findVDom(child);
            if (childVDom) {
                let component: IReactComponent;
                if (childVDom.component) {
                    component = updateReactComponent(childVDom.component);
                } else {
                    component = updateReactComponent(childVDom);
                }
                children.push(component);
            }
        });
        return {
            _currentElement: element,
            _inDevTools: false,
            _renderedChildren: children,
            _stringText: isText ? node.textContent : null,
            node,
        };
    }
    const instanceMap = new Map();
    /**
     * 将vdom或component转换为实例
     * @param componentOrVDom
     */
    function updateReactComponent(componentOrVDom: any): IReactComponent {
        const isVDom = findVDom(componentOrVDom) == null;
        const newInstance = isVDom ? createReactDOMComponent(componentOrVDom) : createReactCompositeComponent(componentOrVDom);
        const base: Element | Component<any, any> = isVDom ? findDOMNode(componentOrVDom) : componentOrVDom;
        if (instanceMap.has(base)) {
            const inst = instanceMap.get(base);
            (Object as any).assign(inst, newInstance);
            return inst;
        }
        instanceMap.set(base, newInstance);
        return newInstance;
    }

    function nextRootKey(roots: IKeyValue) {
        return "." + Object.keys(roots).length;
    }

    function findRoots(node: Element, roots: IKeyValue) {
        Array.prototype.forEach.call(node.childNodes, function _(child: any) {
            const vdom = findVDom(child);
            if (vdom && vdom.component) {
                roots[nextRootKey(roots)] = updateReactComponent(vdom.component);
            } else {
                findRoots(child, roots);
            }
        });
    }

    function isRootComponent(component: Component<IKeyValue, IKeyValue>) {
        if (component._parentComponent) {
            return false;
        }
        const base = findDOMNode(component);
        const parentElement: any = base && base.parentElement;
        const vdom = findVDom(parentElement);
        if (vdom && vdom.props) {
            return false;
        }
        return true;
    }

    function visitNonCompositeChildren(component: IReactComponent, visitor: (arg: IReactComponent) => void) {
        if (component._renderedComponent) {
            if (!component._renderedComponent._component) {
                visitor(component._renderedComponent);
                visitNonCompositeChildren(component._renderedComponent, visitor);
            }
        } else if (component._renderedChildren) {
            component._renderedChildren.forEach(function _(child) {
                visitor(child);
                if (!child._component) {
                    visitNonCompositeChildren(child, visitor);
                }
            });
        }
    }

    function createDevToolsBridge(vdom?: IVDom) {
        const ComponentTree = {
            getClosestInstanceFromNode: function getClosestInstanceFromNode(node: any) {
                while (node && !findVDom(node)) {
                    node = node.parentNode;
                }
                return node ? updateReactComponent((findVDom(node) as IVDom).component) : null;
            },
            getNodeFromInstance: function getNodeFromInstance(instance: IReactComponent) {
                return instance.node;
            },
        };

        const roots: {
            [name: string]: IReactComponent;
        } = {};
        if (vdom && vdom.component) {
            roots[".0"] = updateReactComponent(vdom.component);
        } else {
            findRoots(document.body, roots);
        }
        // findRoots(document.body, roots);

        const Mount = {
            _instancesByReactRootID: roots,
            _renderNewRootComponent: function _renderNewRootComponent(arg: IReactComponent) {},
        };
        const Reconciler = {
            mountComponent: function mountComponent(arg: IReactComponent) {},
            performUpdateIfNecessary: function performUpdateIfNecessary() {},
            receiveComponent: function receiveComponent(arg: IReactComponent) {},
            unmountComponent: function unmountComponent(arg: IReactComponent) {},
        };
        const componentAdded = function componentAdded_(component: Component<IKeyValue, IKeyValue>) {
            const instance = updateReactComponent(component);
            if (isRootComponent(component)) {
                instance._rootID = nextRootKey(roots);
                roots[instance._rootID] = instance;
                Mount._renderNewRootComponent(instance);
            }
            visitNonCompositeChildren(instance, function _(childInst) {
                childInst._inDevTools = true;
                Reconciler.mountComponent(childInst);
            });
            Reconciler.mountComponent(instance);
        };
        const componentUpdated = function componentUpdated_(component: Component<IKeyValue, IKeyValue>) {
            const prevRenderedChildren: IReactComponent[] = [];
            visitNonCompositeChildren(instanceMap.get(component), function _(childInst: IReactComponent) {
                prevRenderedChildren.push(childInst);
            });
            const instance = updateReactComponent(component);
            Reconciler.receiveComponent(instance);
            visitNonCompositeChildren(instance, function _(childInst: IReactComponent) {
                if (!childInst._inDevTools) {
                    childInst._inDevTools = true;
                    Reconciler.mountComponent(childInst);
                } else {
                    Reconciler.receiveComponent(childInst);
                }
            });
            prevRenderedChildren.forEach(function _(childInst: IReactComponent) {
                if (childInst.node && !document.body.contains(childInst.node)) {
                    instanceMap.delete(childInst.node);
                    Reconciler.unmountComponent(childInst);
                }
            });
        };
        const componentRemoved = function componentRemoved_(component: Component<IKeyValue, IKeyValue>) {
            const instance = updateReactComponent(component);
            visitNonCompositeChildren(instance, function _(childInst) {
                instanceMap.delete(childInst.node);
                Reconciler.unmountComponent(childInst);
            });
            Reconciler.unmountComponent(instance);
            instanceMap.delete(component);
            if (instance._rootID) {
                delete roots[instance._rootID];
            }
        };
        return {
            ComponentTree,
            Mount,
            Reconciler,
            componentAdded,
            componentRemoved,
            componentUpdated,
        };
    }

    return function initDevTools(vdom?: IVDom) {
        if (typeof window.__REACT_DEVTOOLS_GLOBAL_HOOK__ === "undefined") {
            return;
        }
        const bridge = createDevToolsBridge(vdom);
        const nextAfterMount = opt.afterMount;
        opt.afterMount = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentAdded(component);
            if (nextAfterMount) {
                nextAfterMount(component);
            }
        };
        const nextAfterUpdate = opt.afterUpdate;
        opt.afterUpdate = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentUpdated(component);
            if (nextAfterUpdate) {
                nextAfterUpdate(component);
            }
        };
        const nextBeforeUnmount = opt.beforeUnmount;
        opt.beforeUnmount = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentRemoved(component);
            if (nextBeforeUnmount) {
                nextBeforeUnmount(component);
            }
        };
        // Notify devtools about this instance of "React"
        window.__REACT_DEVTOOLS_GLOBAL_HOOK__.inject(bridge);
        return () => {
            opt.afterMount = nextAfterMount;
            opt.afterUpdate = nextAfterUpdate;
            opt.beforeUnmount = nextBeforeUnmount;
        };
    };
}

==> zreact-0.3.2.tgz: package/src/devtools-base.ts <==
import { options, Component, findDOMNode as IfindDOMNode, findVDom as IfindVDom} from "./zreact";
import { IKeyValue } from "./types";
import { IVDom } from "./vdom/index";

interface IReactElement {
    props?: boolean|IKeyValue;
    type: any;
    ref?: ((c: Component<IKeyValue, IKeyValue>|null) => void)|null;
    key?: string;
}
interface IReactComponent {
    _currentElement: IReactElement|string|null;
    node?: Text|Element|Node;
    _instance?: Component<IKeyValue, IKeyValue>;
    _renderedComponent?: IReactComponent;
    forceUpdate?: () => void;
    getName?: () => void;
    props?: IKeyValue;
    setState?: () => void;
    state?: IKeyValue;
    _inDevTools?: boolean;
    _renderedChildren?: IReactComponent[];
    _stringText?: null|string;
    _rootID?: string;
    _component?: any;
}

declare const window: {
    $zreact: IVDom,
    __REACT_DEVTOOLS_GLOBAL_HOOK__?: any,
};

declare class Map {
    public has(key: any): boolean;
    public get(key: any): any;
    public set(key: any, val: any): void;
    public delete(key: any): void;
}

export function getInitDevTools(opt: typeof options, findDOMNode: typeof IfindDOMNode, findVDom: typeof IfindVDom) {

    /**
     * 将zreact的component实例转换为React的
     * @param  component
     */
    function createReactElement(component: Component<IKeyValue, IKeyValue>): IReactElement {
        const element: IReactElement = {
            key: component._key,
            props: component.props,
            ref: null, // Unsupported
            type: component.constructor,
        };
        return element;
    }

    /**
     * 获取组件名
     * @param  element
     */
    function typeName(element: IReactElement): string {
        if (typeof element.type === "function") {
            return element.type.displayName || element.type.name;
        }
        return element.type;
    }
    /**
     * 将component转换实例
     * @param component
     */
    function createReactCompositeComponent(component: Component<IKeyValue, IKeyValue>): IReactComponent {
        const _currentElement = createReactElement(component);
        const node = findDOMNode(component);
        const instance: IReactComponent = {
            _currentElement,
            _instance: component,
            forceUpdate: component.forceUpdate && component.forceUpdate.bind(component),
            getName: function getName() {
                return typeName(_currentElement);
            },
            node,
            props: component.props,
            setState: component.setState && component.setState.bind(component),
            state: component.state,
        };
        if (component._component) {
            instance._renderedComponent = updateReactComponent(component._component);
        } else if (findVDom(component)) {
            instance._renderedComponent = updateReactComponent(findVDom(component));
        }
        return instance;
    }

    /**
     * 将vdom转换为实例
     * @param vdom
     */
    function createReactDOMComponent(vdom: IVDom): IReactComponent {
        const node = findDOMNode(vdom);
        const childNodes = node.nodeType === Node.ELEMENT_NODE ?
            Array.prototype.slice.call(node.childNodes, 0) : [];
        const isText = node.nodeType === Node.TEXT_NODE;
        let element: string|IReactElement|null;
        if (isText) {
            element = node.textContent;
        } else {
            element = {
                props: vdom.props,
                type: node.nodeName.toLowerCase(),
            };
        }
        const children: IReactComponent[] = new Array();
        childNodes.forEach(function _(child: any) {
            const childVDom = findVDom(child);
            if (childVDom) {
                let component: IReactComponent;
                if (childVDom.component) {
                    component = updateReactComponent(childVDom.component);
                } else {
                    component = updateReactComponent(childVDom);
                }
                children.push(component);
            }
        });
        return {
            _currentElement: element,
            _inDevTools: false,
            _renderedChildren: children,
            _stringText: isText ? node.textContent : null,
            node,
        };
    }
    const instanceMap = new Map();
    /**
     * 将vdom或component转换为实例
     * @param componentOrVDom
     */
    function updateReactComponent(componentOrVDom: any): IReactComponent {
        const isVDom = findVDom(componentOrVDom) == null;
        const newInstance = isVDom ? createReactDOMComponent(componentOrVDom) : createReactCompositeComponent(componentOrVDom);
        const base: Element | Component<any, any> = isVDom ? findDOMNode(componentOrVDom) : componentOrVDom;
        if (instanceMap.has(base)) {
            const inst = instanceMap.get(base);
            (Object as any).assign(inst, newInstance);
            return inst;
        }
        instanceMap.set(base, newInstance);
        return newInstance;
    }

    function nextRootKey(roots: IKeyValue) {
        return "." + Object.keys(roots).length;
    }

    function findRoots(node: Element, roots: IKeyValue) {
        Array.prototype.forEach.call(node.childNodes, function _(child: any) {
            const vdom = findVDom(child);
            if (vdom && vdom.component) {
                roots[nextRootKey(roots)] = updateReactComponent(vdom.component);
            } else {
                findRoots(child, roots);
            }
        });
    }

    function isRootComponent(component: Component<IKeyValue, IKeyValue>) {
        if (component._parentComponent) {
            return false;
        }
        const base = findDOMNode(component);
        const parentElement: any = base && base.parentElement;
        const vdom = findVDom(parentElement);
        if (vdom && vdom.props) {
            return false;
        }
        return true;
    }

    function visitNonCompositeChildren(component: IReactComponent, visitor: (arg: IReactComponent) => void) {
        if (component._renderedComponent) {
            if (!component._renderedComponent._component) {
                visitor(component._renderedComponent);
                visitNonCompositeChildren(component._renderedComponent, visitor);
            }
        } else if (component._renderedChildren) {
            component._renderedChildren.forEach(function _(child) {
                visitor(child);
                if (!child._component) {
                    visitNonCompositeChildren(child, visitor);
                }
            });
        }
    }

    function createDevToolsBridge(vdom?: IVDom) {
        const ComponentTree = {
            getClosestInstanceFromNode: function getClosestInstanceFromNode(node: any) {
                while (node && !findVDom(node)) {
                    node = node.parentNode;
                }
                return node ? updateReactComponent((findVDom(node) as IVDom).component) : null;
            },
            getNodeFromInstance: function getNodeFromInstance(instance: IReactComponent) {
                return instance.node;
            },
        };

        const roots: {
            [name: string]: IReactComponent;
        } = {};
        if (vdom && vdom.component) {
            roots[".0"] = updateReactComponent(vdom.component);
        } else {
            findRoots(document.body, roots);
        }
        // findRoots(document.body, roots);

        const Mount = {
            _instancesByReactRootID: roots,
            _renderNewRootComponent: function _renderNewRootComponent(arg: IReactComponent) {},
        };
        const Reconciler = {
            mountComponent: function mountComponent(arg: IReactComponent) {},
            performUpdateIfNecessary: function performUpdateIfNecessary() {},
            receiveComponent: function receiveComponent(arg: IReactComponent) {},
            unmountComponent: function unmountComponent(arg: IReactComponent) {},
        };
        const componentAdded = function componentAdded_(component: Component<IKeyValue, IKeyValue>) {
            const instance = updateReactComponent(component);
            if (isRootComponent(component)) {
                instance._rootID = nextRootKey(roots);
                roots[instance._rootID] = instance;
                Mount._renderNewRootComponent(instance);
            }
            visitNonCompositeChildren(instance, function _(childInst) {
                childInst._inDevTools = true;
                Reconciler.mountComponent(childInst);
            });
            Reconciler.mountComponent(instance);
        };
        const componentUpdated = function componentUpdated_(component: Component<IKeyValue, IKeyValue>) {
            const prevRenderedChildren: IReactComponent[] = [];
            const node = findDOMNode(component);
            if (!instanceMap.has(node)) {
                componentAdded(component);
                return;
            }
            visitNonCompositeChildren(instanceMap.get(node), function _(childInst: IReactComponent) {
                prevRenderedChildren.push(childInst);
            });
            const instance = updateReactComponent(component);
            Reconciler.receiveComponent(instance);
            visitNonCompositeChildren(instance, function _(childInst: IReactComponent) {
                if (!childInst._inDevTools) {
                    childInst._inDevTools = true;
                    Reconciler.mountComponent(childInst);
                } else {
                    Reconciler.receiveComponent(childInst);
                }
            });
            prevRenderedChildren.forEach(function _(childInst: IReactComponent) {
                if (childInst.node && !document.body.contains(childInst.node)) {
                    instanceMap.delete(childInst.node);
                    Reconciler.unmountComponent(childInst);
                }
            });
        };
        const componentRemoved = function componentRemoved_(component: Component<IKeyValue, IKeyValue>) {
            const instance = updateReactComponent(component);
            visitNonCompositeChildren(instance, function _(childInst) {
                instanceMap.delete(childInst.node);
                Reconciler.unmountComponent(childInst);
            });
            Reconciler.unmountComponent(instance);
            instanceMap.delete(component);
            if (instance._rootID) {
                delete roots[instance._rootID];
            }
        };
        return {
            ComponentTree,
            Mount,
            Reconciler,
            componentAdded,
            componentRemoved,
            componentUpdated,
        };
    }

    return function initDevTools(vdom?: IVDom) {
        if (typeof window.__REACT_DEVTOOLS_GLOBAL_HOOK__ === "undefined") {
            return;
        }
        const bridge = createDevToolsBridge(vdom);
        const nextAfterMount = opt.afterMount;
        opt.afterMount = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentAdded(component);
            if (nextAfterMount) {
                nextAfterMount(component);
            }
        };
        const nextAfterUpdate = opt.afterUpdate;
        opt.afterUpdate = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentUpdated(component);
            if (nextAfterUpdate) {
                nextAfterUpdate(component);
            }
        };
        const nextBeforeUnmount = opt.beforeUnmount;
        opt.beforeUnmount = (component: Component<IKeyValue, IKeyValue>) => {
            bridge.componentRemoved(component);
            if (nextBeforeUnmount) {
                nextBeforeUnmount(component);
            }
        };
        // Notify devtools about this instance of "React"
        window.__REACT_DEVTOOLS_GLOBAL_HOOK__.inject(bridge);
        return () => {
            opt.afterMount = nextAfterMount;
            opt.afterUpdate = nextAfterUpdate;
            opt.beforeUnmount = nextBeforeUnmount;
        };
    };
}
